.include "m16def.inc"				;  AtMega16A
;*************************************************************************
;*************************************************************************
;    RAM ()
.DSEG
	.equ 		XTAL = 8000000 		;   
	.equ 		BaudRate = 19200	;   UART
	.equ 		BaudDivider = XTAL/(16*BaudRate)-1

	.equ		bit0	= 0b00000001
	.equ		bit1	= 0b00000010
	.equ		bit2	= 0b00000100
	.equ		bit3	= 0b00001000
	.equ		bit4	= 0b00010000
	.equ		bit5	= 0b00100000
	.equ		bit6	= 0b01000000
	.equ		bit7	= 0b10000000	


	.def		ZReg	= R15		;  
	
	Params:			.byte 	4		;   P,I,D     - PWM Base
	Pwm:			.byte	3		;   - ,      - PWM Saw
	
	SensValues:		.byte	7		;    
	CalValues:		.byte	7		;   

;     (Flash)
.CSEG
.ORG 0

	;	   -
	;	PA0-PA6 -->   
	;	PB0-PB6 -->   
	;	PB7 -->  
	;	PC0 --> Ultra Sonic Trigger
	;	PC1 --> Ultra Sonic Echo
	;	PD2 --> DIR2 Left
	;	PD3 --> PWM2 Left
	;	PD4 --> PWM1 Right
	;	PD5 --> DIR1 Right

;*************************************************************************
;*************************************************************************
;  . 
;       Timer0
	JMP 	RESET 					; Reset Handler
	JMP 	0x00	; EXT_INT0 		; IRQ0 Handler
	JMP 	0x00	; EXT_INT1 		; IRQ1 Handler	
	JMP 	0x00	; TIM2_COMP 	; Timer2 Compare Handler
	JMP 	0x00	; TIM2_OVF 		; Timer2 Overflow Handler
	JMP 	0x00	; TIM1_CAPT 	; Timer1 Capture Handler
	JMP 	0x00	; TIM1_COMPA 	; Timer1 CompareA Handler
	JMP 	0x00	; TIM1_COMPB 	; Timer1 CompareB Handler
	JMP 	0x00	; TIM1_OVF 		; Timer1 Overflow Handler
	JMP 	TIM0_OVF 				; Timer0 Overflow Handler
	JMP 	0x00	; SPI_STC 		; SPI Transfer Complete Handler
	JMP 	0x00	; USART_RXC 	; USART RX Complete Handler
	JMP 	0x00	; USART_UDRE 	; UDR Empty Handler
	JMP 	0x00	; USART_TXC 	; USART TX Complete Handler
	JMP 	0x00	; ADCC 			; ADC Conversion Complete Handler
	JMP 	0x00	; EE_RDY 		; EEPROM Ready Handler
	JMP 	0x00	; ANA_COMP 		; Analog Comparator Handler
	JMP 	0x00	; TWI 			; Two-wire Serial Interface Handler
	JMP 	0x00	; EXT_INT2 		; IRQ2 Handler
	JMP 	0x00	; TIM0_COMP 	; Timer0 Compare Handler
	JMP 	0x00	; SPM_RDY 		; Store Program Memory Ready Handler
;*************************************************************************
;*************************************************************************
;      Timer0
TIM0_OVF:
	;     R16,R17,ZH,ZL  SREG
	;   
	PUSH	R16
	LDI 	R16,0
	OUT 	TCNT0,R16
	IN		R16, SREG		
	PUSH	R16
	PUSH	R17	
	PUSH	ZH
	PUSH	ZL
	
	;    
	CALL	PWM_drive

	;     R16,R17,ZH,ZL  SREG
	POP		ZL
	POP		ZH
	POP		R17
	POP		R16
	OUT		SREG, R16	
	POP		R16
RETI 
;*************************************************************************
;*************************************************************************
;  
RESET:
;        (RAM)
	LDI 	R16,HIGH(RAMEND)
	OUT 	SPH,R16 			
	LDI 	R16,LOW(RAMEND)
	OUT 	SPL,R16


;   R0-R31
	LDI		ZL, 0x1E			;    	
	CLR		ZH					;      
	DEC		ZL					;  
	ST		Z, ZH				;    0
	BRNE	PC-2				;      
;  
	LDI		ZL,Low(SRAM_START)	;     
	LDI		ZH,High(SRAM_START)
	CLR		R16					;  R16
Flush:		
	ST 		Z+,R16				;  0   
	CPI		ZH,High(RAMEND+1)	;   ?
	BRNE	Flush				; ?  !
	CPI		ZL,Low(RAMEND+1)	;     ?
	BRNE	Flush
	CLR	ZL						;  
	CLR	ZH
;   
	CLI
;   -
	LDI		R16,0xFF
	LDI		R17,bit7
	EOR		R16,R17
	OUT 	DDRB,R16
	LDI		R16,(1<<2)|(1<<3)|(1<<4)|(1<<5)
	OUT 	DDRD,R16
	LDI		R16,(1<<2)|(1<<5)
	OUT 	PORTD,R16
	LDI		R16,(1<<0)
	OUT		DDRC,R16
	LDI		R16,(0<<1)
	OUT		PORTC,R16	
;   -   2.56
	LDI 	R16,(1<<REFS0)|(1<<REFS1)|(1<<ADLAR)
	OUT 	ADMUX,R16
;   - (UART)
	CALL	UART_init
;  Timer0.   8MHz.  /8 (84 . ) 
;    ,   
	LDI		R16,(0<<CS00)|(1<<CS01)|(0<<CS02)
	OUT		TCCR0, R16
	LDI		R16,(0<<TOIE0)
	OUT		TIMSK,R16
;     (PWM Saw Pwm+2)
	LDI		R16,0xFF
	STS		Pwm+2,R16
;  .    (Pwm)    (Pwm+1)
	LDI		R16,0xFF
	STS		Pwm,R16
	LDI		R16,0xFF
	STS		Pwm+1,R16
;   
	SEI
;*************************************************************************
;*************************************************************************
;   
MAIN:	


UltraSonic:
;   	
	LDI		R16,26					; 80  = 10 = N*3+2=80.  N=26
	SBI		PORTC,0
Trig:
	DEC		R16
	BRNE	Trig
	CBI		PORTC,0		;   Trigger
	CLR		YL
	CLR		YH
Echo:
	SBIS	PINC,1 					;   Echo
	RJMP	Echo
;   
EchoCount:
	ADIW	Y,1						; 2 
	SBIC    PINC, 1     			; 1/2 : false=1, true=2
	RJMP	EchoCount				; 2 

	;   M=(2+1+2)*(N-1)+2+2+1( SBIS )=5*N
	;   V=340,29 /c
	;    = M*5/8*(V/2/1000)=M*5/8*0,17=M/32/5*17

;	MOV		R16,YH
;	RCALL	UART_snt
;	MOV		R16,YL
;	RCALL	UART_snt
;	RCALL	UART_delay

	MOVW	R16,YL
;   5
	LDI		R18,5
	CLR		R19
	RCALL	DIV16U
;   4
	LSR		R17
	ROR		R16
	LSR		R17
	ROR		R16
;   17
	CLR		R18
	CLR		R19
	LDI		R20,17

	MUL		R16,R20	
	MOVW	R18,R0
	MUL		R17,R20
	ADD		R18,R0
	ADC		R19,R1
;   8
	LSR		R19
	ROR		R18
	LSR		R19
	ROR		R18
	LSR		R19
	ROR		R18


	MOV		R16,R19
	RCALL	UART_snt
	MOV		R16,R18
	RCALL	UART_snt
	RCALL	UART_delay




;  P-

;   
	CALL	Scan_sensors

;      
	CLR		R16
	CLR		R17
	CLR		R18
	CLR		R19
;        
	LDI		YL,low(SensValues)
	LDI		YH,high(SensValues)
	LDI		R20,1
StartCalc:
	LD		R22,Y+
	MOV		R21,R20
	
StartCalcSum:
;  R17:R16   * 	
	ADD		R16,R22
	ADC		R17,ZReg	
	DEC		R21
	BRNE	StartCalcSum
;  R19:R18   
	ADD		R18,R22
	ADC		R19,ZReg	
	INC		R20
	CPI		R20,8
	BRNE	StartCalc
;    
	MOVW	R2,R18


;  
	RCALL	DIV16U
;     (  ,   3<E<5)
	MOV		R10,R16
;    10
	MOVW	R16,R0
	LDI		R20,10
	MUL		R16,R20	
	MOVW	R18,R0
	MUL		R17,R20
	ADD		R18,R0
	ADC		R19,R1

	MOVW	R16,R18
;    
	MOVW	R18,R2


;  
	RCALL	DIV16U
;     (  ,   3<E<5)
	MOV		R11,R16
;    10
	MOVW	R16,R0
	LDI		R20,10
	MUL		R16,R20	
	MOVW	R18,R0
	MUL		R17,R20
	ADD		R18,R0
	ADC		R19,R1
	
	MOVW	R16,R18
;    
	MOVW	R18,R2
	

;  
	RCALL	DIV16U
;     (  ,   3<E<5)
	MOV		R12,R16

	MOV		R20,R10
	MOV		R21,R11
	MOV		R22,R12
;     	
	CPI		R22,5
	BRLO	Memp
	INC		R21
Memp:
	CPI		R20,4
	BREQ	Memp2

	LDI		R23,10
	SUB		R23,R21	
	LDI		R18,1
	JMP		Memp3
	
Memp2:		
	MOV		R23,R21
	LDI		R18,2

Memp3:
	LDI		R16,5 		; p-gain
	LDI		R17,0x32	; pwm base
	
	MOV		R19,R16	
	LSL		R19
	LSL		R19
	
	CLR		R20
	
MMM:
	ADD		R20,R19
	DEC		R23
	BRNE	MMM

	ADD		R20,R17		

RegLeft:
	CPI		R18,1 ;left
	BRNE	RegRight
	STS		pwm+1,R20
	STS		pwm,R17
	
	JMP		MainNext

RegRight:
	STS		pwm,R20
	STS		pwm+1,R17



	JMP		Main

MainNext:
	;      Timer0
	LDI		R16,(1<<TOIE0)
	OUT		TIMSK,R16
	

JMP		Main


;*************************************************************************
;*************************************************************************
;  
; PWM Saw    255  0
;     PWM Saw   1,  0
;     Channel/255
;    1        
PWM_drive:
	IN		R17, PORTD		;  
	CBR		R17, bit3+bit4	;   1  2

	LDS		ZL,Pwm			;   Left
	LDS		ZH,Pwm+1		;   Right
	LDS		R16,Pwm+2		;   PWM Saw

	CP		R16, ZL			;   
	BRCC	PC+2			;  ZL<R16
	SBR		R17, bit3		;  ZL>R16   
	
	CP		R16, ZH			;   
	BRCC	PC+2			;  ZH<R16
	SBR		R17, bit4		;  ZH>R16   

	OUT		PORTD, R17		;  R17  
	SUBI	R16, 1			;    
	BRCC	PC+2			;  R16>0	
	LDI		R16, 0xFF		;    
							
	STS		Pwm+2, R16
RET
;*************************************************************************
;*************************************************************************
;  
;   Sensors    
;   R19 -    . 
;   R18 - . 
;    0  7  
Scan_sensors:
	LDI		R16,0x60	;   
	LDI		R17,1<<0	;  
	CLR		R21

	LDI		YL,low(SensValues)
	LDI		YH,high(SensValues)

Start_scan:

	;   
	OUT		PORTB,R17
	;    
	OUT		ADMUX, R16
	
	LDI		R21,16
	CLR		R18			;    
	CLR		R19			;    


GetSum:	
	;    
	LDI		R20,(1<<ADEN)|(0<<ADIE)|(1<<ADSC)|(0<<ADATE)|(5<<ADPS0)
	OUT		ADCSRA,R20
	;   
	SBIC	ADCSRA,ADSC
	RJMP	PC-1
	;   
	IN		R20,ADCH	

	ADD		R18,R20
	ADC		R19,ZReg
	
	DEC		R21
	BRNE	GetSum
	LDI		R21,4
GetDiv:
	LSR		R19
	ROR		R18

	DEC		R21
	BRNE	GetDiv

	;RCALL	UART_snt
	;PUSH	R16
	;MOV		R16,R18
	;RCALL	UART_snt
	;POP		R16
	ST		Y+,R18
	;RCALL	UART_delay
Next_scan:				

	;    
	INC		R16
	;   
	LSL		R17
	CPI		R16,0x67
	BRNE	Start_scan

RET	
;*************************************************************************
;*************************************************************************
;  UART
UART_init:
	PUSH	R16	
	LDI 	R16,LOW(BaudDivider)
	OUT 	UBRRL,R16
	LDI 	R16, HIGH(BaudDivider)
	OUT 	UBRRH,R16
	LDI 	R16,0
	OUT 	UCSRA, R16
	;    ,   
	LDI 	R16, (1<<RXEN)|(1<<TXEN)|(0<<RXCIE)|(0<<TXCIE)|(0<<UDRIE)
	OUT 	UCSRB, R16	
	;   : 8  , 1  ,   
	LDI 	R16, (1<<URSEL)|(1<<UCSZ0)|(1<<UCSZ1)
	OUT 	UCSRC, R16
	POP		R16
RET
;    R16  UART
UART_snt:	
	SBIS 	UCSRA,UDRE	;     
	RJMP	UART_snt 	;   -  UDRE
	OUT		UDR, R16
RET
;    UART      R16
UART_rcv:	
	SBIS	UCSRA,RXC	;    
	RJMP	UART_rcv	;   
	IN		R16,UDR		;   - .
RET
;*************************************************************************
;*************************************************************************
;   0.6 
UART_Delay:
	LDI		R20, $25
WGLOOP0:  
	LDI  	R21, $B8
WGLOOP1:  
	LDI  	R22, $EA
WGLOOP2:  
	DEC  	R22
    BRNE 	WGLOOP2
    DEC  	R21
    BRNE 	WGLOOP1
    DEC  	R20
    BRNE 	WGLOOP0
    LDI  	R20, $53
WGLOOP3:  
	DEC  	R20
    BRNE 	WGLOOP3
RET
;*************************************************************************
;*************************************************************************
;   2- 
; R17:R16	 (   )
; R19:R18	
; R1:R0		
DIV16U:	
	CLR		R0
	CLR		R1
	LDI		R20,16
	LSL		R16
	ROL		R17
	ROL		R0
	ROL		R1
	CP		R0,R18
	CPC		R1,R19
	BRCS	PC+4
	INC		R16
	SUB		R0,R18
	SBC		R1,R19
	DEC		R20
	BRNE	PC-11
RET


